@@ -1,3 +1,4 @@
+package KinoSearch::FieldType;
 use KinoSearch;
 
 1;
@@ -10,7 +11,7 @@ my $synopis = <<'END_SYNOPSIS';
     # abstract base class
 END_SYNOPSIS
 
-Boilerplater::Binding::Perl::Class->register(
+Clownfish::Binding::Perl::Class->register(
     parcel       => "KinoSearch",
     class_name   => "KinoSearch::FieldType",
     bind_methods => [
@@ -44,7 +45,7 @@ Boilerplater::Binding::Perl::Class->register(
 
 __COPYRIGHT__
 
-Copyright 2005-2009 Marvin Humphrey
+Copyright 2005-2010 Marvin Humphrey
 
 This program is free software; you can redistribute it and/or modify
 under the same terms as Perl itself.
