@@ -1,5 +1,10 @@
+package KinoSearch::Test;
 use KinoSearch;
 
+# Set the default memory threshold for PostingListWriter to a low number so
+# that we simulate large indexes by performing a lot of PostingPool flushes.
+KinoSearch::Index::PostingListWriter::set_default_mem_thresh(0x1000);
+
 1;
 
 __END__
@@ -12,7 +17,7 @@ MODULE = KinoSearch   PACKAGE = KinoSearch::Test::TestUtils
 SV*
 doc_set()
 CODE:
-    KOBJ_TO_SV_NOINC( kino_TestUtils_doc_set(), RETVAL );
+    RETVAL = KINO_OBJ_TO_SV_NOINC(kino_TestUtils_doc_set());
 OUTPUT: RETVAL
 
 MODULE = KinoSearch   PACKAGE = KinoSearch::Test
@@ -48,13 +53,19 @@ PPCODE:
     else if (strEQ(package, "TestNumericType")) {
         kino_TestNumericType_run_tests();
     }
-    /* KinoSearch::Obj */
+    /* KinoSearch::Object */
     else if (strEQ(package, "TestObj")) {
         kino_TestObj_run_tests();
     }
+    else if (strEQ(package, "TestI32Array")) {
+        kino_TestI32Arr_run_tests();
+    }
     else if (strEQ(package, "TestByteBuf")) {
         kino_TestBB_run_tests();
     }
+    else if (strEQ(package, "TestLockFreeRegistry")) {
+        kino_TestLFReg_run_tests();
+    }
     /* KinoSearch::QueryParser */
     else if (strEQ(package, "TestQueryParserSyntax")) {
     }
@@ -68,8 +79,11 @@ PPCODE:
     else if (strEQ(package, "TestDocWriter")) {
         kino_TestDocWriter_run_tests();
     }
-    else if (strEQ(package, "TestPostingsWriter")) {
-        kino_TestPostWriter_run_tests();
+    else if (strEQ(package, "TestPostingListWriter")) {
+        kino_TestPListWriter_run_tests();
+    }
+    else if (strEQ(package, "TestSegment")) {
+        kino_TestSeg_run_tests();
     }
     else if (strEQ(package, "TestSegWriter")) {
         kino_TestSegWriter_run_tests();
@@ -113,10 +127,49 @@ PPCODE:
         kino_TestTermQuery_run_tests();
     }
     /* KinoSearch::Store */
+    else if (strEQ(package, "TestCompoundFileReader")) {
+        kino_TestCFReader_run_tests();
+    }
+    else if (strEQ(package, "TestCompoundFileWriter")) {
+        kino_TestCFWriter_run_tests();
+    }
+    else if (strEQ(package, "TestFileHandle")) {
+        kino_TestFH_run_tests();
+    }
+    else if (strEQ(package, "TestFolder")) {
+        kino_TestFolder_run_tests();
+    }
+    else if (strEQ(package, "TestFSDirHandle")) {
+        kino_TestFSDH_run_tests();
+    }
+    else if (strEQ(package, "TestFSFolder")) {
+        kino_TestFSFolder_run_tests();
+    }
+    else if (strEQ(package, "TestFSFileHandle")) {
+        kino_TestFSFH_run_tests();
+    }
     else if (strEQ(package, "TestInStream")) {
         kino_TestInStream_run_tests();
     }
+    else if (strEQ(package, "TestIOChunks")) {
+        kino_TestIOChunks_run_tests();
+    }
+    else if (strEQ(package, "TestIOPrimitives")) {
+        kino_TestIOPrimitives_run_tests();
+    }
+    else if (strEQ(package, "TestRAMDirHandle")) {
+        kino_TestRAMDH_run_tests();
+    }
+    else if (strEQ(package, "TestRAMFileHandle")) {
+        kino_TestRAMFH_run_tests();
+    }
+    else if (strEQ(package, "TestRAMFolder")) {
+        kino_TestRAMFolder_run_tests();
+    }
     /* KinoSearch::Util */
+    else if (strEQ(package, "TestAtomic")) {
+        kino_TestAtomic_run_tests();
+    }
     else if (strEQ(package, "TestBitVector")) {
         kino_TestBitVector_run_tests();
     }
@@ -126,6 +179,12 @@ PPCODE:
     else if (strEQ(package, "TestHash")) {
         kino_TestHash_run_tests();
     }
+    else if (strEQ(package, "TestJson")) {
+        kino_TestJson_run_tests();
+    }
+    else if (strEQ(package, "TestIndexFileNames")) {
+        kino_TestIxFileNames_run_tests();
+    }
     else if (strEQ(package, "TestNumberUtils")) {
         kino_TestNumUtil_run_tests();
     }
@@ -135,6 +194,9 @@ PPCODE:
     else if (strEQ(package, "TestPriorityQueue")) {
         kino_TestPriQ_run_tests();
     }
+    else if (strEQ(package, "TestStringHelper")) {
+        kino_TestStrHelp_run_tests();
+    }
     else if (strEQ(package, "TestMemoryPool")) {
         kino_TestMemPool_run_tests();
     }
@@ -153,7 +215,9 @@ run_tests(index);
     kino_Folder *index;
 PPCODE:
     kino_TestQPSyntax_run_tests(index);
+END_XS_CODE
 
+my $charm_xs_code = <<'END_XS_CODE';
 MODULE = KinoSearch   PACKAGE = KinoSearch::Test::TestCharmonizer
 
 void
@@ -165,25 +229,25 @@ PPCODE:
     chaz_Test_init();
 
     if (strcmp(which, "dirmanip") == 0) {
-        batch = chaz_TDirManip_prepare();
+        batch = chaz_TestDirManip_prepare();
     }
     else if (strcmp(which, "integers") == 0) {
-        batch = chaz_TIntegers_prepare();
+        batch = chaz_TestIntegers_prepare();
     }
     else if (strcmp(which, "func_macro") == 0) {
-        batch = chaz_TFuncMacro_prepare();
+        batch = chaz_TestFuncMacro_prepare();
     }
     else if (strcmp(which, "headers") == 0) {
-        batch = chaz_THeaders_prepare();
+        batch = chaz_TestHeaders_prepare();
     }
     else if (strcmp(which, "large_files") == 0) {
-        batch = chaz_TLargeFiles_prepare();
+        batch = chaz_TestLargeFiles_prepare();
     }
     else if (strcmp(which, "unused_vars") == 0) {
-        batch = chaz_TUnusedVars_prepare();
+        batch = chaz_TestUnusedVars_prepare();
     }
     else if (strcmp(which, "variadic_macros") == 0) {
-        batch = chaz_TVariadicMacros_prepare();
+        batch = chaz_TestVariadicMacros_prepare();
     }
     else {
         THROW(KINO_ERR, "Unknown test identifier: '%s'", which);
@@ -194,16 +258,27 @@ PPCODE:
 }
 END_XS_CODE
 
-Boilerplater::Binding::Perl::Class->register(
+Clownfish::Binding::Perl::Class->register(
     parcel            => "KinoSearch",
     class_name        => "KinoSearch::Test::TestSchema",
-    xs_code           => $xs_code,
     bind_constructors => ["new"],
 );
 
+Clownfish::Binding::Perl::Class->register(
+    parcel            => "KinoSearch",
+    class_name        => "KinoSearch::Test",
+    xs_code           => $xs_code,
+);
+
+Clownfish::Binding::Perl::Class->register(
+    parcel            => "KinoSearch",
+    class_name        => "KinoSearch::Test::TestCharmonizer",
+    xs_code           => $charm_xs_code,
+);
+
 __COPYRIGHT__
 
-Copyright 2005-2009 Marvin Humphrey
+Copyright 2005-2010 Marvin Humphrey
 
 This program is free software; you can redistribute it and/or modify
 under the same terms as Perl itself.
